// bibliotheque bit-bangded UART
// l'utilisateur doit configurer les ports pour les bits RX et TX
// crer les temporisations de 1 bit et 1/2 bit en fonction de la vitesse en BAUD et de FOSC

#include <p18cxxx.h>
#include "bb_uart.h"

// PORTS bit-banged UART
#define BBTXD         PORTBbits.RB1           // TX
#define TRIS_BBTXD    TRISBbits.TRISB1        // TRIS TX
#define BBRXD         PORTBbits.RB2           // RX
#define TRIS_BBRXD    TRISBbits.TRISB2        // TRIS RX

// Avec Q=32MHz TCY=125nS
// Pour une transmission  9600 BAUD 
// t1 bit = 104uS   104/0.125 = 832 cycles
// t1/2 bit = 52us  52/0.125 = 416 cycles
// A ajuster avec des temporisations, et compter
// "environ" 20 cycles pour l'appel et le retour d'une fonction
// la tempo bit est ainsi ramenee a 770 cycles, le 1/2 bit a 400 
// Il est indispensable de controler la duree du bit en sortie avec un oscilloscope

// tempo 1 bit
void tempoBitUART(void)
{
	Delay100TCYx(7);
	Delay10TCYx(7);
}

// tempo 1/2 bit
void tempoDemiBitUART(void)
{
	Delay100TCYx(4);
}

// initialise les PORTS RX TX
void initBBUART(void)
{
    TRIS_BBTXD=0;
    BBTXD=1;
    TRIS_BBRXD=1;
}


// lit un octet sur RX en BB
unsigned char litBBUART(void)
{
unsigned char cptbit=8;
unsigned char data=0;

	while (BBRXD);
	tempoDemiBitUART();	// attend milieu start bit
	while (cptbit)
	{
		tempoBitUART();
		data>>=1;
		if (BBRXD) data|=0b10000000;
		cptbit--;
	}
	tempoBitUART();		// attend fin stop bit
	return data;
}

// envoie un octet sur TX en BB
void ecritBBUART (unsigned char data)
{
unsigned char cptbit=8;
	BBTXD=0;
	tempoBitUART(); // start bit
	while (cptbit)
	{
		BBTXD=((data) & 0x01);
		data>>=1;
		tempoBitUART();
		cptbit--;
	}
	BBTXD=1;
	tempoBitUART(); // stop bit
}
